{
    "dspRelease": "test instance",
    "apiProtocol": "http",
    "apiHost": "{{ CUSTOM_HOST }}",
    "apiPort": 3333,
    "apiPath": "",
    "iiifProtocol": "http",
    "iiifHost": "{{ CUSTOM_HOST }}",
    "iiifPort": 1024,
    "iiifPath": "",
    "ingestUrl": "http://{{ CUSTOM_HOST }}:3340",
    "geonameToken": "knora",
    "jsonWebToken": "",
    "logErrors": false,
    "iriBase": "http://rdfh.ch",
    "instrumentation": {
      "environment": "production",
      "rollbar": {
        "enabled": false,
        "accessToken": ""
      }
    },
    "featureFlags": {
      "allowEraseProjects": true
    }
}
